from edc_form_validators import FormValidator


class FbgFormValidatorMixin:
    """Declare with FormValidatorMixin, modelform"""

    def validate_fbg_required_fields(self: FormValidator, fbg_prefix: str):
        """Uses fields `fbg_value`, `fbg_datetime`, `fbg_units`.

        Args:
            :param fbg_prefix: e.g. fbg, fbg2, etc
        """

        self.invalid_if_before_report_datetime(
            f"{fbg_prefix}_datetime",
            report_datetime_field=self.report_datetime_field_attr,
        )
        self.required_if_true(
            self.cleaned_data.get(f"{fbg_prefix}_datetime"),
            field_required=f"{fbg_prefix}_value",
        )

        self.required_if_true(
            self.cleaned_data.get(f"{fbg_prefix}_value"),
            field_required=f"{fbg_prefix}_units",
        )

        self.required_if_true(
            self.cleaned_data.get(f"{fbg_prefix}_value"),
            field_required=f"{fbg_prefix}_datetime",
        )



from django.core.exceptions import ValidationError
from django.test import TestCase
from edc_constants.constants import NOT_APPLICABLE, YES
from edc_lab.constants import EQ
from edc_reportable import MILLIMOLES_PER_LITER
from edc_utils import get_utcnow

from edc_glucose.form_validators import GlucoseFormValidator

from ..models import Appointment, SubjectVisit


class TestGlucose(TestCase):
    def setUp(self):
        self.subject_identifier = "1234"
        appointment = Appointment.objects.create(
            subject_identifier=self.subject_identifier, visit_code=1000, visit_code_sequence=0
        )
        self.subject_visit_baseline = SubjectVisit.objects.create(
            appointment=appointment,
            subject_identifier=self.subject_identifier,
            visit_code=1000,
            visit_code_sequence=0,
        )
        appointment = Appointment.objects.create(
            subject_identifier=self.subject_identifier, visit_code=2000, visit_code_sequence=0
        )
        self.subject_visit_followup = SubjectVisit.objects.create(
            appointment=appointment,
            subject_identifier=self.subject_identifier,
            visit_code=2000,
            visit_code_sequence=0,
        )

    def test_glucose_result(self):
        class MyGlucoseFormValidator(GlucoseFormValidator):
            def clean(self):
                self.validate_glucose_test()

        cleaned_data = dict(
            subject_visit=self.subject_visit_baseline,
            glucose_performed=YES,
            glucose_units=MILLIMOLES_PER_LITER,
        )
        form_validator = MyGlucoseFormValidator(cleaned_data=cleaned_data)
        with self.assertRaises(ValidationError) as cm:
            form_validator.validate()
        self.assertIn("glucose_date", cm.exception.error_dict)

        cleaned_data = dict(
            subject_visit=self.subject_visit_baseline,
            glucose_performed=YES,
            glucose_date=get_utcnow().date,
            glucose_units=MILLIMOLES_PER_LITER,
        )
        form_validator = MyGlucoseFormValidator(cleaned_data=cleaned_data)
        with self.assertRaises(ValidationError) as cm:
            form_validator.validate()
        self.assertIn("fasting", cm.exception.error_dict)

        cleaned_data = dict(
            subject_visit=self.subject_visit_baseline,
            glucose_performed=YES,
            glucose_date=get_utcnow().date,
            fasting=YES,
            glucose_units=MILLIMOLES_PER_LITER,
        )
        form_validator = MyGlucoseFormValidator(cleaned_data=cleaned_data)
        with self.assertRaises(ValidationError) as cm:
            form_validator.validate()
        self.assertIn("glucose_value", cm.exception.error_dict)

        cleaned_data = dict(
            subject_visit=self.subject_visit_baseline,
            glucose_performed=YES,
            glucose_date=get_utcnow().date,
            fasting=YES,
            glucose_value=5.3,
            glucose_units=MILLIMOLES_PER_LITER,
            glucose_quantifier=None,
        )
        form_validator = MyGlucoseFormValidator(cleaned_data=cleaned_data)
        with self.assertRaises(ValidationError) as cm:
            form_validator.validate()
        self.assertIn("glucose_quantifier", cm.exception.error_dict)

        cleaned_data = dict(
            subject_visit=self.subject_visit_baseline,
            glucose_performed=YES,
            glucose_date=get_utcnow().date,
            fasting=YES,
            glucose_value=5.3,
            glucose_units=NOT_APPLICABLE,
            glucose_quantifier=EQ,
        )
        form_validator = MyGlucoseFormValidator(cleaned_data=cleaned_data)
        with self.assertRaises(ValidationError) as cm:
            form_validator.validate()
        self.assertIn("glucose_units", cm.exception.error_dict)

        cleaned_data = dict(
            subject_visit=self.subject_visit_baseline,
            glucose_performed=YES,
            glucose_date=get_utcnow().date,
            fasting=YES,
            glucose_value=5.3,
            glucose_units=MILLIMOLES_PER_LITER,
            glucose_quantifier=EQ,
        )
        form_validator = MyGlucoseFormValidator(cleaned_data=cleaned_data)
        try:
            form_validator.validate()
        except ValidationError:
            self.fail("ValidationError unexceptedly raised")



from __future__ import annotations

from django import forms

from ..utils import validate_glucose_as_millimoles_per_liter
from .fbg_form_validator_mixin import FbgFormValidatorMixin
from .ogtt_form_validator_mixin import OgttFormValidatorMixin


class FbgOgttFormValidatorMixin(FbgFormValidatorMixin, OgttFormValidatorMixin):
    def validate_glucose_testing_matrix(
        self,
        ogtt_prefix: str | None = None,
        fbg_prefix: str | None = None,
        include_fbg: bool | None = None,
    ) -> None:
        ogtt_prefix = ogtt_prefix or "ogtt"
        fbg_prefix = fbg_prefix or "fbg"

        include_fbg = True if include_fbg is None else include_fbg
        if include_fbg:
            self.validate_fbg_required_fields(fbg_prefix)
            validate_glucose_as_millimoles_per_liter(fbg_prefix, self.cleaned_data)
        self.validate_ogtt_required_fields(ogtt_prefix=ogtt_prefix)
        validate_glucose_as_millimoles_per_liter(ogtt_prefix, self.cleaned_data)
        self.validate_ogtt_dates(ogtt_prefix=ogtt_prefix)
        self.validate_fbg_before_ogtt(ogtt_prefix=ogtt_prefix, fbg_prefix=fbg_prefix)
        self.validate_ogtt_time_interval()

    def validate_fbg_before_ogtt(
        self, ogtt_prefix: str | None = None, fbg_prefix: str | None = None
    ):
        """Validate the FBG is performed before the OGTT"""
        ogtt_prefix = ogtt_prefix or "ogtt"
        fbg_prefix = fbg_prefix or "fbg"
        fbg_dte = self.cleaned_data.get(f"{fbg_prefix}_datetime")
        ogtt_base_dte = self.cleaned_data.get(f"{ogtt_prefix}_base_datetime")
        if fbg_dte and ogtt_base_dte:
            total_seconds = (ogtt_base_dte - fbg_dte).total_seconds()
            if total_seconds <= 1:
                raise forms.ValidationError(
                    {
                        f"{ogtt_prefix}_base_datetime": (
                            "Invalid date. Expected to be after time FBG level was measured"
                        )
                    }
                )



from edc_constants.constants import YES

from ..utils import validate_glucose_as_millimoles_per_liter


class GlucoseFormValidatorMixin:
    fasting_fld = "fasting"

    def validate_glucose_test(self) -> None:
        self.required_if(YES, field="glucose_performed", field_required="glucose_date")
        self.required_if(YES, field="glucose_performed", field_required=self.fasting_fld)
        self.required_if(YES, field="glucose_performed", field_required="glucose_value")
        validate_glucose_as_millimoles_per_liter("glucose", self.cleaned_data)
        self.required_if(YES, field="glucose_performed", field_required="glucose_quantifier")
        self.required_if(YES, field="glucose_performed", field_required="glucose_units")
